clear all
close all

%% README
% This scrpit is aim to help myself to test some basic parameters in
% seismic receiver function (Ps conversion only) migrition works. Briefly,
% the script calculates the S wave traveltime field as well as related
% angles first. Then, there is a forward modeling part to calculate the
% traveltime of the direct P wave, the Pms wave and the Pds wave, plus
% their polarities. After that, in migration part, the delay times of the 
% Pms and Pds wave are convoluted with given wavelet and then migrated to
% corresponding positions. Finally, the polting section shows the results
% of normalized migrated profile.

% Note one: the amplitude of the receiver function calcuated in this script
% is not accurate. add density information and some transmission
% coefficients if you need them to be precise.

% Note two: only scatter coefficients are taken into account here. In
% practice, effects from geometical spreading and attenuation may also need
% to be added.



%% Model Setting
% interface 1
model.moho_depth   = 40.0;  % depth of moho (interface 1) in km
model.moho_dipping = 10.0;  % dipping angle of moho in degrees
% interface 2
model.disc_depth   = 60.0;  % depth of discontinuity (interface 2) in km
model.disc_dipping = 30.0;  % dipping angle of discontinuity in degrees
% velocity structures
model.Vp_crust     = 6.0;   % Vp of first layer in km/s
model.Vs_crust     = 3.42;  % Vs of first layer in km/s
model.Vp_mantle    = 8.0;   % Vp of second layer
model.Vs_mantle    = 4.45;  % Vs of second layer
model.Vp_bottom    = 10.0;  % Vp of third (bottom) layer
model.Vs_bottom    = 5.5;   % Vs of third (bottom) layer


%% Initialization 

% stations
station_spacing = 5;
x_station = 10:station_spacing:290;

% parameters of migration area
dist_vec  = 0:1:300;
depth_vec = 0:1:200;


% model parameters
moho_depth   = model.moho_depth;
moho_dipping = model.moho_dipping;
disc_depth   = model.disc_depth;
disc_dipping = model.disc_dipping;

Vp_crust  = model.Vp_crust;
Vs_crust  = model.Vs_crust ; 
Vp_mantle = model.Vp_mantle;
Vs_mantle = model.Vs_mantle;
Vp_bottom = model.Vp_bottom;
Vs_bottom = model.Vs_bottom;


%% S wave traveltime field ( very slow !! only need to be calculated once if none of the parameters above is changed)

traveltime_field_S = zeros(length(depth_vec)*length(dist_vec),5,length(x_station));

poolobj = parpool('local',4);
parfor ista = 1:length(x_station)
    traveltime_field_S(:,:,ista) = raytracing_three_layers(model, dist_vec(end), depth_vec(end), x_station(ista));
end
delete(poolobj)


%% Forward Modeling + Migration

potential = zeros(length(depth_vec), length(dist_vec));

% type and width of the wavelet
gauss_or_ricker = 0;    % 1 for gauss and others for ricker

if (gauss_or_ricker == 1)
    width_gauss = 0.1;
else
    freq_ricker = 2.0;
end

% amplication factor of the Pms wave and Pds wave
amp_Pms = 1.0;
amp_Pds = 1.0;


% loop for slowness
for islw = [-8:-4 4:8]
        
    %% Forward Modeling
    % Calculating the traveltime of the direct P wave, the Pms wave and the
    % Pds wave. Moreover, the polarity of the Pms and Pds waves are also
    % calculated using transimision coefficients
    
    slowness = 0.01 * islw;    % slowness of plane wave in second layer
    theta_P = asind(slowness*Vp_bottom);    % incidence of P wave in the bottom layer
    
       
    % traveltime of the direct P wave (DP)
    alpha_disc_DP = asind(sind(disc_dipping - theta_P)*Vp_mantle/Vp_bottom);
    alpha_moho_DP = asind(sind(moho_dipping - disc_dipping + alpha_disc_DP)*Vp_crust/Vp_mantle);
    
    x_moho_DP = ( x_station - moho_depth * tand(moho_dipping - alpha_moho_DP) )/( 1 + tand(moho_dipping)*tand(moho_dipping - alpha_moho_DP));
    d_moho_DP = x_moho_DP * tand(moho_dipping) + moho_depth;
    
    x_disc_DP = ( x_moho_DP + d_moho_DP * tand(disc_dipping-alpha_disc_DP) - disc_depth * tand(disc_dipping-alpha_disc_DP))/(1 + tand(disc_dipping)*tand(disc_dipping - alpha_disc_DP));
    d_disc_DP = x_disc_DP * tand(disc_dipping) + disc_depth;
    
    time_p1_DP = -(x_disc_DP * sind(disc_dipping - theta_P))/(Vp_bottom * cosd(disc_dipping));
    time_p2_DP = sqrt((x_moho_DP - x_disc_DP).^2 + (d_moho_DP - d_disc_DP).^2)/Vp_mantle;
    time_p3_DP = sqrt((x_station - x_moho_DP).^2 + (d_moho_DP).^2)/Vp_crust;
    
    traveltime_DP = time_p1_DP + time_p2_DP + time_p3_DP;
    
    
    % traveltime of the Ps wave at moho (Pms)
    beta_moho_Pms = asind(sind(moho_dipping - disc_dipping + alpha_disc_DP)*Vs_crust/Vp_mantle);
    
    x_moho_Pms = ( x_station - moho_depth * tand(moho_dipping - beta_moho_Pms) )/( 1 + tand(moho_dipping)*tand(moho_dipping - beta_moho_Pms));
    d_moho_Pms = x_moho_Pms * tand(moho_dipping) + moho_depth;
    
    x_disc_Pms = ( x_moho_Pms + d_moho_Pms * tand(disc_dipping-alpha_disc_DP) - disc_depth * tand(disc_dipping-alpha_disc_DP))/(1 + tand(disc_dipping)*tand(disc_dipping - alpha_disc_DP));
    d_disc_Pms = x_disc_Pms * tand(disc_dipping) + disc_depth;
    
    time_p1_Pms = -(x_disc_Pms * sind(disc_dipping - theta_P))/(Vp_bottom * cosd(disc_dipping));
    time_p2_Pms = sqrt((x_moho_Pms - x_disc_Pms).^2 + (d_moho_Pms - d_disc_Pms).^2)/Vp_mantle;
    time_s3_Pms = sqrt((x_station - x_moho_Pms).^2 + (d_moho_Pms).^2)/Vs_crust;
    
    sign_Pms = sign( slowness * -sind( 2*(moho_dipping - beta_moho_Pms - (disc_dipping - alpha_disc_DP)) ) );
    traveltime_Pms = time_p1_Pms + time_p2_Pms + time_s3_Pms;
    
    
    % traveltime of the Ps wave at discontinuity (Pds)   
    beta_disc_Pds = asind(sind(disc_dipping - theta_P)*Vs_mantle/Vp_bottom);
    beta_moho_Pds = asind(sind(moho_dipping - disc_dipping + beta_disc_Pds)*Vs_crust/Vs_mantle);
    
    x_moho_Pds = ( x_station - moho_depth * tand(moho_dipping - beta_moho_Pds) )/( 1 + tand(moho_dipping)*tand(moho_dipping - beta_moho_Pds));
    d_moho_Pds = x_moho_Pds * tand(moho_dipping) + moho_depth;
    
    x_disc_Pds = ( x_moho_Pds + d_moho_Pds * tand(disc_dipping-beta_disc_Pds) - disc_depth * tand(disc_dipping-beta_disc_Pds))/(1 + tand(disc_dipping)*tand(disc_dipping - beta_disc_Pds));
    d_disc_Pds = x_disc_Pds * tand(disc_dipping) + disc_depth;
    
    time_p1_Pds = -(x_disc_Pds * sind(disc_dipping - theta_P))/(Vp_bottom * cosd(disc_dipping));
    time_s2_Pds = sqrt((x_moho_Pds - x_disc_Pds).^2 + (d_moho_Pds - d_disc_Pds).^2)/Vs_mantle;
    time_s3_Pds = sqrt((x_station - x_moho_Pds).^2 + (d_moho_Pds).^2)/Vs_crust;
    
    sign_Pds = sign( slowness * -sind( 2*(disc_dipping - beta_disc_Pds - theta_P) ) * cosd( 2*(moho_dipping - beta_moho_Pms - (disc_dipping - beta_disc_Pds)) ) );
    traveltime_Pds = time_p1_Pds + time_s2_Pds + time_s3_Pds;
    
 
    % diff time between Ps conversions (Pms, Pds) and direct P wave
    diff_time_moho = traveltime_Pms - traveltime_DP;
    diff_time_disc = traveltime_Pds - traveltime_DP;


    %% Migration

    % loop for stations
    for ista = 1:length(x_station)        
        for idep = 1:length(depth_vec)
            depth_loop = depth_vec(idep) + 0.5;
            for idist = 1:length(dist_vec)
                dist_loop = dist_vec(idist) + 0.5;
                
                % cal_time_angle_P function returns the travletime of the P and the angle of ray vector at given scatter point            
                [time_P, angle_P] = cal_time_angle_P(model, dist_loop, depth_loop, slowness);
                
                index_S = idist + (idep-1)*length(dist_vec);
                time_S  = traveltime_field_S(index_S, 3, ista);
                
                if (time_S < 0)         % if it is fail to find the S wave traveltime, then use nearby points to interpolate it
                    tmp_time_S  = 0.0;
                    tmp_angle_S = 0.0;
                    tmp_incid_S = 0.0;
                    counts = 0.0;
                    
                    for ii = -1:1
                        for jj = -1:1
                            tmp_index = (idist + jj) + (idep + ii - 1)*length(dist_vec);
                            if (traveltime_field_S(tmp_index, 3, ista) < 0)
                                
                                tmp_time_S  = tmp_time_S  + traveltime_field_S(tmp_index, 3, ista);
                                tmp_angle_S = tmp_angle_S + traveltime_field_S(tmp_index, 4, ista);
                                tmp_incid_S = tmp_incid_S + traveltime_field_S(tmp_index, 5, ista);
                                counts = counts + 1.0;
                            end
                        end
                    end
                    
                    time_S  =  tmp_time_S/counts;
                    angle_S = tmp_angle_S/counts;
                    incid_S = tmp_incid_S/counts;
                     
                else                    
                    angle_S = traveltime_field_S(index_S, 4, ista);
                    incid_S = traveltime_field_S(index_S, 5, ista);
                end
                
                
                traveltime_loop = time_P + time_S;
                angle_scatter = angle_S - angle_P;
                
                difftime_loop = traveltime_loop - traveltime_DP(ista);
                scatter_coefficient = -1.0 * sind(2*angle_scatter);
                station_correction  = sign(slowness) * cosd(incid_S);
                
%                 % distance between scatter and station, may be used for taking geometrical spreading effect into account later
%                 distance_SX = sqrt((dist_loop - x_station(ista))^2 + depth_loop^2);
                
                if (gauss_or_ricker == 1)
%                     % receiver function with random noise (not the correct way to add noise but may give some inspiration)
%                     rf = 1.0 * gaussmf(difftime_loop,[width_gauss diff_time_moho(ista)]) + 1.5 * gaussmf(difftime_loop,[width_gauss diff_time_disc(ista)]) + 0.2*(rand(1)-0.5);
                    rf = amp_Pms * sign_Pms * gaussmf(difftime_loop,[width_gauss diff_time_moho(ista)]) + amp_Pds * sign_Pds * gaussmf(difftime_loop,[width_gauss diff_time_disc(ista)]);
                else
                    rf = amp_Pms * sign_Pms * ricker(difftime_loop-diff_time_moho(ista), freq_ricker) + amp_Pds * sign_Pds * ricker(difftime_loop-diff_time_disc(ista), freq_ricker);
                end
                
                % stacking
                potential(idep,idist) = potential(idep, idist) + station_correction * scatter_coefficient * rf;
                
            end
        end
    end
    
end   %end of loop for slowenss 



%% Ploting 

figure(4)
normalized_fac = max(max(abs(potential)));
imagesc(dist_vec,depth_vec,potential./normalized_fac)
colorbar
width_height_ratio = dist_vec(end)/depth_vec(end);
set(gcf,'Units','centimeters','InnerPosition',[5 5 20*[width_height_ratio 1]])
caxis([-1 1])
xlabel('distance(km)')
ylabel('depth(km)')

figure(5)
imagesc(dist_vec,depth_vec,potential./normalized_fac)
colorbar
hold on
plot(dist_vec,tand(moho_dipping).*dist_vec+moho_depth,'r')
plot(dist_vec,tand(disc_dipping).*dist_vec+disc_depth,'r')
set(gcf,'Units','centimeters','InnerPosition',[5 5 20*[width_height_ratio 1]])
caxis([-1 1])
xlabel('distance(km)')
ylabel('depth(km)')
hold off

% print(['three_layers_mig_result_no_ref__slw_0.04_0.08.jpeg'],figure(4),'-djpeg','-r900')
% print(['three_layers_mig_result_with_ref_slw_0.04_0.08.jpeg'],figure(5),'-djpeg','-r900')
% print(['mig_result_no_ref_slw_0.04_0.12.jpeg'],figure(4),'-djpeg','-r900')
% print(['mig_result_with_ref_slw_0.04_0.12.jpeg'],figure(5),'-djpeg','-r900')
% print(['mig_result_no_ref_slw_0.09_0.12.jpeg'],figure(4),'-djpeg','-r900')
% print(['mig_result_with_ref_slw_0.09_0.12.jpeg'],figure(5),'-djpeg','-r900')
