function traveltime_field = raytracing_three_layers(model, dist_max, depth_max, x_station)
% S wave raytracing in the three layers model.
% This function calculates the S wave traveltime from the station to every
% grid in the migrated area. The top two layers adopt analytic solution to
% calculate the traveltime whilt the bottom layer use the bisection method 
% to get the numeric result.


    % model parameters
    moho_depth   = model.moho_depth;
    moho_dipping = model.moho_dipping;
    disc_depth   = model.disc_depth;
    disc_dipping = model.disc_dipping;

%    Vp_crust  = model.Vp_crust;
    Vs_crust  = model.Vs_crust ; 
%    Vp_mantle = model.Vp_mantle;
    Vs_mantle = model.Vs_mantle;
%    Vp_bottom = model.Vp_bottom;
    Vs_bottom = model.Vs_bottom;
    
    index = 1;
    x_dist_vec  = 0:1:dist_max;
    d_depth_vec = 0:1:depth_max;

    traveltime_field = zeros(length(d_depth_vec)*length(x_dist_vec),5);


    for ii = 1:length(d_depth_vec)
        d_depth_g = d_depth_vec(ii) + 0.5;
        for jj = 1:length(x_dist_vec)
            x_dist_g = x_dist_vec(jj) + 0.5;
            
            if (d_depth_g < (x_dist_g*tand(moho_dipping) + moho_depth) )                   
                % grid inside the first layer
                
                time_S  = sqrt((x_dist_g - x_station)^2 + (d_depth_g)^2)/Vs_crust;
                angle_S = atand((x_station - x_dist_g)/d_depth_g);
                incid_S = angle_S;
                traveltime_field(index,:) = [x_dist_g d_depth_g time_S angle_S incid_S];
                index = index+1;
                
            else
                
                if (d_depth_g < (x_dist_g*tand(disc_dipping) + disc_depth) )   
                    % grid inside the second layer
                    
                    tmp_depth = moho_depth + x_dist_g*tand(moho_dipping);
                            
                    % rotate to flat interface
                    dist_g = (d_depth_g - tmp_depth) * cosd(moho_dipping);
                    dist_x = (moho_depth + x_station*tand(moho_dipping)) * cosd(moho_dipping);
                    length_gx = (x_station - x_dist_g)/cosd(moho_dipping) - dist_g * tand(moho_dipping) - dist_x * tand(moho_dipping);


                    C4 = Vs_crust^2 - Vs_mantle^2;
                    C3 = -2*length_gx*C4;
                    C2 = Vs_crust^2 * (dist_x^2 + length_gx^2) - Vs_mantle^2 * (dist_g^2 + length_gx^2);
                    C1 = 2*length_gx*Vs_mantle^2*dist_g^2;
                    C0 = -1*Vs_mantle^2*length_gx^2*dist_g^2;

                    x_solu = roots([C4 C3 C2 C1 C0]);
                    x_inter = -1;
                    for kk = 1:length(x_solu)
                        if ( ~ isreal(x_solu(kk)))
                            continue;
                        end
                        if ( (x_solu(kk) >= 0 && x_solu(kk) <= length_gx) || (x_solu(kk) <= 0 && x_solu(kk) >= length_gx) )
                            x_inter = x_dist_g + (dist_g * sind(moho_dipping) + x_solu(kk))*cosd(moho_dipping);
                            break;
                        end
                    end
                    
                    
                    if( x_inter == -1 )            
                        
                        time_S = -1.0;
                        angle_S = NaN;
                        incid_S = NaN;
                        
                        traveltime_field(index,:) = [x_dist_g depth_tmp time_S angle_S incid_S]; 
                        
                    else                        
                        d_inter = x_inter * tand(moho_dipping) + moho_depth;
                        
                        time_S = sqrt((d_depth_g - d_inter)^2 + (x_dist_g - x_inter)^2)/Vs_mantle + sqrt((x_inter - x_station)^2 + (d_inter)^2)/Vs_crust;
                        angle_S = asind( (x_inter - x_dist_g)/(sqrt( (d_depth_g - d_inter)^2 + (x_dist_g - x_inter)^2) ) );
                        incid_S = atand((x_station - x_inter)/d_inter);
                        traveltime_field(index,:) = [x_dist_g d_depth_g time_S angle_S incid_S];
                        
                    end
                    
                    index = index+1;
                
                else
                    % grid inside the third layer
                    
                    upper_x_limit = x_dist_vec(end);
                    lower_x_limit = x_dist_vec(1);
                    
                    x_tmp = x_dist_g;
                    while( (upper_x_limit - lower_x_limit) > 0.05*abs(x_tmp - x_dist_g) )
                        
                        % if the raypath is ideally vertical, break from here
                        if ( (upper_x_limit - lower_x_limit) < 1e-6 )
                            x_tmp = x_dist_g;
                            d_tmp = x_tmp*tand(disc_dipping) + disc_depth;
                            theta_below = -disc_dipping;
                            
                            break;
                        end
                        
                        % bisection method to find x_tmp and x_inter
                        x_tmp = 0.5*(upper_x_limit + lower_x_limit);
                        d_tmp = x_tmp*tand(disc_dipping) + disc_depth;
                        
                        theta_below = asind( (x_tmp - x_dist_g)/(sqrt( (d_depth_g - d_tmp)^2 + (x_dist_g - x_tmp)^2) ) ) - disc_dipping;
                        
                        
                        % copy from for two layer model
                        tmp_depth = moho_depth + x_tmp*tand(moho_dipping);
                            
                        % rotate to flat interface
                        dist_g = (d_tmp - tmp_depth) * cosd(moho_dipping);
                        dist_x = (moho_depth + x_station*tand(moho_dipping)) * cosd(moho_dipping);
                        length_gx = (x_station - x_tmp)/cosd(moho_dipping) - dist_g * tand(moho_dipping) - dist_x * tand(moho_dipping);


                        C4 = Vs_crust^2 - Vs_mantle^2;
                        C3 = -2*length_gx*C4;
                        C2 = Vs_crust^2 * (dist_x^2 + length_gx^2) - Vs_mantle^2 * (dist_g^2 + length_gx^2);
                        C1 = 2*length_gx*Vs_mantle^2*dist_g^2;
                        C0 = -1*Vs_mantle^2*length_gx^2*dist_g^2;
                        
                        x_solu = roots([C4 C3 C2 C1 C0]);
                        x_inter = -1;
                        for kk = 1:length(x_solu)
                            if ( ~ isreal(x_solu(kk)))
                                continue;
                            end
                            if ( (x_solu(kk) >= 0 && x_solu(kk) <= length_gx) || (x_solu(kk) <= 0 && x_solu(kk) >= length_gx) )
                                x_inter = x_tmp + (dist_g * sind(moho_dipping) + x_solu(kk))*cosd(moho_dipping);
                                break;
                            end
                        end
                        
                        % if fail to find the x_inter
                        if( x_inter == -1 )            
                            break;
                        end
                        
                        d_inter = x_inter * tand(moho_dipping) + moho_depth;
                        
                        theta_above = asind( (x_inter - x_tmp)/(sqrt( (d_inter - d_tmp)^2 + (x_inter - x_tmp)^2) ) ) - disc_dipping;
                        
                        if ( sind(theta_above) * Vs_bottom > sind(theta_below) * Vs_mantle )
                            lower_x_limit = x_tmp;
                        else
                            upper_x_limit = x_tmp;
                        end
                        
                    
                    end
                    
                    
                    
                    
                    if (x_inter == -1)          % unable to find the numerical solution

                        time_S = -1.0;
                        angle_S = NaN;
                        incid_S = NaN;

                        traveltime_field(index,:) = [x_dist_g depth_tmp time_S angle_S incid_S];
                        index = index+1;
                    else
                        
                        time_S_3 = sqrt((x_dist_g - x_tmp)^2 + ( d_depth_g - d_tmp)^2)/Vs_bottom;
                        time_S_2 = sqrt((x_tmp - x_inter)^2 + (d_tmp - d_inter)^2)/Vs_mantle;
                        time_S_1 = sqrt((x_inter - x_station)^2 + (d_inter)^2)/Vs_crust;

                        time_S = time_S_3 + time_S_2 + time_S_1;

                        angle_S = theta_below + disc_dipping;
                        incid_S = atand((x_station - x_inter)/d_inter);
                        traveltime_field(index,:) = [x_dist_g d_depth_g time_S angle_S incid_S];
                        index = index+1;
                    end
                end
                
            end
        end
    end
    
    return;
end